
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_CLIENT_CLASS_H
#define PG_CLIENT_CLASS_H

#include "xstd/AlarmClock.h"
#include "xstd/Ring.h"
#include "runtime/XactFarm.h"
#include "runtime/Agent.h"
#include "runtime/BcastRcver.h"
#include "client/IcpCltXact.h"
#include "client/PrivWorld.h"

class Connection;
class CltConnMgr;
class CltCfg;
class PortMgr;
class ServerRep;
class ObjId;
class OidRec;
class PrivCache;
class DistrPoint;
class RobotSym;

class Client: public Agent, public AlarmUser, public BcastRcver {
	public:
		static void Farm(XactFarm<CltXact> *aFarm);
		static void LogState(OLog &log);
		static ServerRep *GetServerRep(int idx);

	public:
		Client();
		virtual ~Client();

		virtual void configure(const RobotSym *cfg, const NetAddr &aHost, bool bind2iface);
		void icpClient(IcpClient *anIcpClient);

		virtual void start();

		// called by the Cache on a miss
		CltXact *fetch(const ObjId &oid, DistrPoint *dp);

		virtual void noteXactDone(CltXact *x);
		virtual void noteEmbedded(CltXact *x, const OidRec &embedOidRec);
		virtual void notePeerAsked(IcpCltXact *q);

		IcpClient *icpClient() { return theIcpClient; }
		PrivCache *privCache() { return thePrivCache; }

	protected:
		bool cAddServer(const NetAddr &addr, bool forceAdd);
		static void BuildWarmupPlan();
		static void FreezeSrvWss();

		PortMgr *getPortMgr(bool bind2iface);
		void scanServers(const RobotSym *rs);
		CltCfg *sharedConfig(const RobotSym *rs);
		bool srvReadyToHit(const ServerRep &s) const;
		int genWarmupSrvIdx();

		void scheduleLaunch(Time lastLaunch);
		bool tryLaunch(CltXact *x);
		void suspendXact(CltXact *x);
		void resumeXact();

		CltXact *genXact(const OidRec &oidRec);
		CltXact *genXact();

		void genOid(OidRec &oidRec);
		void genOid2(OidRec &oidRec);

		const NetAddr &nextHopAddr(const CltXact *x) const;
		void askPeer(const NetAddr &addr, CltXact *x);

		virtual void noteInfoEvent(BcastChannel *ch, InfoEvent ev);
		virtual void noteLogEvent(BcastChannel *ch, OLog &log);

	private:
		bool launch(CltXact *x);

	protected:
		static XactFarm<CltXact> *TheXacts;
		static ObjFarm<IcpCltXact> TheIcpXacts;

		static Array<PortMgr*> ThePortMgrs;  // port managers for all aliases
		static Array<ServerRep*> TheServers; // all servers known to clients
		static Ring<int> *TheWarmupPlan;     // servers to touch during warm-up
		static int TheClientCnt;             // global client counter

		Array<PrivWorld> thePrivWorlds; // per-origin private worlds
		PrivCache *thePrivCache;        // private cache for this agent

		Array<CltXact*> theLaunchDebts; // postponed requests

		CltCfg *theCfg;         // client configuration (maybe shared)
		CltConnMgr *theConnMgr; // one for all servers

		IcpClient *theIcpClient;

		int theCcXactLvl;      // outstanding concurrent xactions level
		int theLaunchCount;    // total number of xaction launches
		int theExtraLaunchLvl; // uncompensated out-of-order launches
};

#endif
