
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl/pgl.h"

#include <math.h>
#include <iostream.h>
#include <strstream.h>

#include "xstd/String.h"
#include "pgl/PglRec.h"
#include "pgl/PglIntSym.h"
#include "pgl/PglNetAddrSym.h"
#include "pgl/PglArraySym.h"
#include "pgl/PglNetAddrRange.h"
#include "pgl/BenchSym.h"


BenchSym::BenchSym(): RecSym("Bench", new PglRec) {
	theRec->bAdd("rate", "peak_req_rate", 0);
	theRec->bAdd("int",  "client_host_count", 0);
	theRec->bAdd("addr", "client_addr_mask", 0);
	theRec->bAdd("addr", "server_addr_mask", 0);
	theRec->bAdd("rate", "max_client_load", 0);
	theRec->bAdd("rate", "max_robot_load", 0);
}

BenchSym::BenchSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool BenchSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Bench";
}

SynSym *BenchSym::clone(const String &type) const {
	if (isA(type))
		return new BenchSym(this->type(), theRec->clone());
	return RecSym::clone(type);
}

bool BenchSym::peakReqRate(double &rate) const {
	return getRate("peak_req_rate", rate);
}

bool BenchSym::clientHostCount(int &count) const {
	return getInt("client_host_count", count);
}

NetAddr BenchSym::cltMask() const {
	return getNetAddr("client_addr_mask");
}

NetAddr BenchSym::srvMask() const {
	return getNetAddr("server_addr_mask");
}

bool BenchSym::maxCltLoad(double &load) const {
	return getRate("max_client_load", load);
}

bool BenchSym::maxRbtLoad(double &load) const {
	return getRate("max_robot_load", load);
}

IntSym *BenchSym::clientHostCount() const {
	int count = -1;
	if (getInt("client_host_count", count))
		return new IntSym(count);

	// calculate, if possible
	double reqRate, cltLoad;
	if (!peakReqRate(reqRate) || !maxCltLoad(cltLoad) || cltLoad <= 0)
		return 0;

	return new IntSym((int)ceil(reqRate/cltLoad));
}

ArraySym *BenchSym::robots() const {
	return sideIps(0);
}

ArraySym *BenchSym::servers() const {
	return sideIps(128);
}

ArraySym *BenchSym::sideIps(int shift) const {
	const NetAddr mask = shift ? srvMask() : cltMask();

	if (!mask)
		return 0;

	double reqRate, cltLoad, rbtLoad;
	if (!peakReqRate(reqRate) || !maxCltLoad(cltLoad) || !maxRbtLoad(rbtLoad))
		return 0;

	const int pairCnt = (int)ceil(reqRate/cltLoad);

	const int rbtCntApx = doubleDiv(pairCnt, reqRate, rbtLoad);
	const int srvCntApx = doubleDiv(pairCnt, rbtCntApx*0.1 + 500, 1);
	const int agentCntApx = shift ? srvCntApx : rbtCntApx;

	const String sideStr = ipStr(mask, pairCnt, agentCntApx, shift);
	return ipStrToArr(sideStr);
}

String BenchSym::ipStr(const NetAddr &mask, int pairCnt, int agentCnt, int shift) const {
	const int maxX = doubleDiv(pairCnt, agentCnt, 250);
	const int maxY = (int)ceil(agentCnt/(double)maxX);

	char bufMask[256];
	ostrstream osMask(bufMask, sizeof(bufMask));
	osMask << mask << ends;

	// leave the "net" part alone
	char *dot1 = strchr(bufMask, '.');
	Assert(dot1);
	char *dot2 = strchr(dot1+1, '.');
	Assert(dot2);
	dot2++;

	ostrstream os(dot2, sizeof(bufMask) - (dot2-bufMask));
	os << (1+shift) << '-' << (maxX+shift) << ".1-" << maxY;

	if (mask.port() >= 0)
		os << ':' << mask.port();

	os << ends;
	return bufMask;
}

ArraySym *BenchSym::ipStrToArr(const String &str) const {
	PglNetAddrRange ar;
	if (!ar.parse(str))
		return 0;

	return ar.toSyms();
}

int BenchSym::doubleDiv(int factor, double n, double d) const {
	const double apx = ceil(n/d);
	return (int)(factor * ceil(apx/factor));
}
