
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_HOSTS_BASED_ITERATOR_H
#define PG_HOSTS_BASED_ITERATOR_H

#include "Array.h"
#include "NetAddr.h"

class HostsBasedSym;

// a configuration of an object often contains 'hosts' array
// the addresses in hosts array should be filtered against "local" ones
// and then coresponding objects should be created based on
// one-for-many configuration and a specific host address
//
// host based iterator hides the complexity of the above process
// the iterator return (object configuration symbol, host) pairs
class HostsBasedIter {
	public:
		typedef Array<HostsBasedSym*> CfgSyms;
		typedef Array<in_addr> Addrs;

	public:
		HostsBasedIter();
		HostsBasedIter(const Addrs *aLocalAddrs);
		virtual ~HostsBasedIter();

		HostsBasedIter &start(const Addrs *localAddrs);
		HostsBasedIter &restart();

		bool eof() const;
		operator void *() const;
		HostsBasedIter &operator ++();

		const NetAddr &host();   // current host
		int hostIdx() const { return theHostIdx; }
		int cfgIdx() const { return theCfgIdx; }

	protected:
		void sync();
		virtual bool belongs(const HostsBasedSym *cfgSym, const NetAddr &host) const;

		virtual HostsBasedSym *cfgSym(int idx) const = 0;
		virtual int cfgCount() const = 0;    // number of cfg symbols

	private:
		NetAddr host(int cidx, int hidx) const;

	protected:
		const Addrs *theLocalAddrs;  // local addresses

		int theCfgIdx;  // sequential cfgSym idx
		int theHostIdx; // sequential host idx within cfgSym

	private:
		NetAddr theHost; // cached value
};

#endif
