
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include <iostream.h>
#include <iomanip.h>

#include "xstd/Rnd.h"
#include "xstd/TblDistr.h"
#include "xstd/gadgets.h"
#include "base/RndPermut.h"
#include "base/ObjId.h"
#include "base/polyLogCats.h"
#include "base/polyLogTags.h"
#include "base/rndDistrStat.h"
#include "runtime/Connection.h"
#include "runtime/HostMap.h"
#include "runtime/ErrorMgr.h"
#include "runtime/LogComment.h"
#include "runtime/StatPhaseMgr.h"
#include "runtime/polyBcastChannels.h"
#include "runtime/polyErrors.h"
#include "runtime/globals.h"
#include "pgl/ServerSym.h"
#include "pgl/SocketSym.h"
#include "server/Server.h"
#include "server/SrvConnMgr.h"
#include "server/SrvOpts.h"


XactFarm<SrvXact> *Server::TheXacts;

void Server::Farm(XactFarm<SrvXact> *aFarm) {
	Assert(!TheXacts && aFarm);
	TheXacts = aFarm;
}

void Server::LogState(OLog &) {
}

Server::Server(): theHostIdx(-1) {
	theConnMgr = new SrvConnMgr(this); // for now

	theChannels.append(TheLogCfgChannel);
	theChannels.append(TheLogStateChannel);
	startListen();
}

Server::~Server() {
	delete theConnMgr;
	deaf();
}

void Server::configure(const ServerSym *cfg, const NetAddr &aHost, bool bind2iface) {
	Assert(theConnMgr);

	SockOpt opt;
	Agent::configure(cfg, aHost, opt);

	theHistory.window(TheSrvOpts.theHistoryWin);
	// XXX: cannot Assert: pxy server does not have a HostMap entry
	TheHostMap->find(aHost, theHostIdx);

	// setup listen socket (XXX: should be moved to start())

	Must(theSock.create());
	Must(theSock.blocking(false));
	Should(theSock.reuseAddr(true));
	Must(theSock.configure(opt));

	if (bind2iface)
		Must(theSock.bind(theHost));
	else
		Must(theSock.bind(NetAddr(0, theHost.port())));

	Must(theSock.listen());

	int acceptLmt = -1;
	cfg->acceptLmt(acceptLmt);
	theConnMgr->configure(opt, cfg->pconnUseLmt());
	theConnMgr->acceptLmt(acceptLmt);
	theConnMgr->idleTimeout(cfg->idlePconnTimeout());

	TheXacts->limit(1024); // magic, no good way to estimate
}

void Server::start() {
	Assert(theSock);
	Assert(theConnMgr);
	Agent::start();
	theReserv = TheFileScanner->setFD(theSock.fd(), dirRead, this);
}

void Server::noteXactDone(SrvXact *x) {
	Assert(x);
	theConnMgr->put(x->conn());
	TheXacts->put(x);
}

void Server::noteReadReady(int) {
	if (!theConnMgr->accept(theSock)) {	// fatal error
		deaf();
		FatalError(errServerGone);
	}
}

void Server::noteConnReady(Connection *conn) {
	startXact(conn); // that's it?!
}

void Server::noteLogEvent(BcastChannel *ch, OLog &log) {
	if (ch == TheLogCfgChannel) {
		log << bege(lgSrvCfg, lgcSrvSide);
		// XXX: implement (SrvCfgRec)
		log << ende;
	} else
	if (ch == TheLogStateChannel) {
		log << bege(lgSrvState, lgcSrvSide);
		log << theSeqvId
			<< theHistory;
		log << ende;
	}
}

void Server::deaf() {
	if (theSock) {
		Comment(2) << "server " << theHost << " is closing listen socket " << theSock.fd() << endc;
		theSock.close();
	}
	if (theReserv)
		TheFileScanner->clearRes(theReserv);
}

void Server::startXact(Connection *conn) {
	SrvXact *x = TheXacts->get();
	x->exec(this, conn,
		theThinkDistr ? Time::Secd(theThinkDistr->trial()) : Time());
}
