
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_SERVER_CLASS_H
#define PG_SERVER_CLASS_H

#include "xstd/Socket.h"
#include "xstd/FileScanner.h"
#include "runtime/Agent.h"
#include "runtime/XactFarm.h"
#include "runtime/BcastRcver.h"
#include "server/CltGroupMap.h"
#include "server/SrvHistory.h"
#include "server/SrvXact.h"
#include "server/SrvConnMgr.h"


class RndDistr;
class OidRec;
class ServerSym;
class ContentSel;


class Server: public Agent, public FileScanner::User, public SrvConnMgr::User, public BcastRcver {
	public:
		static void LogState(OLog &log);
		static void Farm(XactFarm<SrvXact> *aFarm);

	public:
		Server();
		~Server();

		void configure(const ServerSym *cfg, const NetAddr &aHost, bool bind2iface);

		void start();

		// xactions need access to this
		SrvHistory &history() { return theHistory; }
		int hostIdx() const { return theHostIdx; }

		void noteXactDone(SrvXact *x);

		virtual void noteReadReady(int fd);
		virtual void noteConnReady(Connection *conn);

	protected:
		void configureContents(const ServerSym *cfg);
		void startXact(Connection *conn);
		void deaf();

		virtual void noteLogEvent(BcastChannel *ch, OLog &log);

	protected:
		static XactFarm<SrvXact> *TheXacts;
		static CltGroupMap TheCltGroups;

		SrvConnMgr *theConnMgr;
		Socket theSock;

		SrvHistory theHistory;
		int theHostIdx;           // index into the HostMap

		FileScanReserv theReserv;
};

#endif
