
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef HISTORY_CLASS_H
#define HISTORY_CLASS_H

#include "Array.h"
#include "gadgets.h"

// History log: a circular buffer; new entries overwrite old ones
// all external coordinates are relative to "now" or last entry
// for simplicity, assume that at most MAX_INT items are inserted
template <class Item>
class History {
	public:
		History(int aCapacity = 0): theLog(aCapacity), theOffset(0) {}

		void capacity(int aCapacity) { Assert(!theOffset); theLog.stretch(aCapacity); }

		int depth() const { return Min(theOffset, theLog.capacity()); }
		Item at(int off) const { return theLog[(theOffset - off) % theLog.capacity()]; }
		Item operator [](int off) const { return at(off); } 

		void insert(Item i) { theLog[++theOffset % theLog.capacity()] = i; }

	protected:
		Array<Item> theLog;
		int theOffset;      // beginning of a buffer
};

#endif
